-- safedir.lua
-- Lists all .slua files in current directory - together with their output
-- when run (or the error they produce when run).
--
-- An arbitrary timeout of 100k instructions per script is employed to prevent
-- the listing from taking to long and to protect against endless loops in any
-- of the scripts.
--
-- Usage: safedir.lua [-v] [dirname]
--
-- Version: Safe Lua 0.2 (http://safelua.sf.net)
-- by Stefan Reich (www.friedensrevolution.de)
-- Contact: stefan.reich.maker.of.eye@gmail.com
-- This code is public domain!

require "lib_safelua"
require "lfs"

timeout = 100000
dirname = '.'

function parseArguments()
  for i=1, #arg do
    if arg[i] == '-v' then
      verbose = true
    else
      dirname = arg[i]
    end
  end
end

function getOutput(scriptfile)
  sandbox = makeSandbox()
  text = ''
  sandbox.print = function (s)
    text = text..s.."\n"
  end
  status, result = runScript(scriptfile, sandbox, timeout)
  return status, result, text
end

function escape(s)
  -- return (s:gsub('[%-%.%+%[%]%(%)%$%^%%%?%*]','%%%1'):gsub('%z','%%z'))
  return s:gsub('\n', '\\n'):gsub('\r', '\\r'):gsub('"', '\\"')
end

function outputtostring(output)
  s = tostring(#output).." byte"
  if #output > 0 then
    maxlen = 20
    if #output > maxlen then
      output = output:sub(0, maxlen).."..."
    end
    --quoted = string.format("%q", output)
    quoted = '"'..escape(output)..'"'
    s = s.." "..quoted
  end
  return s
end

parseArguments()

print("\n-= listing: *.slua in "..dirname.." =-\n")

for file in lfs.dir(dirname) do
  if file:find('%.slua$') then
    fullfile = dirname..'/'..file
    t = lfs.attributes(fullfile, "mode")
    if t == 'file' then
      print(file)
      status, result, output = getOutput(fullfile)
      if status then
        print("  => outputs "..outputtostring(output))
      elseif #output > 0 then
        print("  => outputs "..tostring(#output).." byte,")
        print("     then fails: "..tostring(result))
      else
        print("  => fails: "..tostring(result))
      end
    end
  end
end
